@@ -1,5 +1,8 @@
 # --------------------------------------------------------------- Imports ---------------------------------------------------------------- #
 
+# System
+from typing import List
+
 # Pip
 from jsoncodable import JSONCodable
 
@@ -15,18 +18,32 @@ class Review(JSONCodable):
 
     def __init__(
         self,
-        dict_: dict
+        id: str,
+        reviewer_name: str,
+        rating: int,
+        upvotes: int,
+        title: str,
+        text: str,
+        language: str,
+        foreign: bool,
+        image_urls: List[str]
     ):
-        self.id = dict_['id']
-        self.review_data = dict_['review_data']
-        self.name = dict_['name']
-        self.rating = dict_['rating']
-        self.title = dict_['title']
-        self.review = dict_['review']
+        self.id = id
+        self.reviewer_name = reviewer_name
+        self.rating = rating
+        self.upvotes = upvotes
+        self.title = title
+        self.text = text
+        self.language = language
+        self.foreign = foreign
+        self.image_urls = image_urls
+
+
+    # ------------------------------------------------------ Public properties ------------------------------------------------------- #
 
     @property
-    def text(self) -> str:
-        return self.review
+    def has_images(self) -> bool:
+        return len(self.image_urls) > 0
 
 
 # ---------------------------------------------------------------------------------------------------------------------------------------- #
\ No newline at end of file
